/* LAN Requester net view replacement for Samba */

call _LoadOtherFuncs

call _InitTempDir

call _SambaInit

call _SambaExtendSearchPath

debuglevel = '-d 0'
samba.!msg = TempDir||'netview.$$$'

parse arg target

if target <> "" then do
    if left(target,2) = '\\' then target = translate(substr(target,3))
end
call charout , 'Please stand by, while I am collectiong data...'

address cmd samba.!smbtreeexe' 'debuglevel' -N >'samba.!msg

call charout , '0D'x||copies(' ',79)||'0D'x

do until lines(samba.!msg) = 0
    smbtreeline = linein(samba.!msg)
    Header = c2x(left(smbtreeline,3))
    select
        when Header = "09095C" then do /* share */
            smbtreeline = strip(smbtreeline,,'09'x)
            parse var smbtreeline '\\'machine'\'share '09'x comment
            machine = strip(machine)
            share   = left(strip(share),13)
            comment = strip(comment)
            text = left(share' 'comment,79)
            if target = machine then say text
        end
        when Header = "095C5C" then do /* Machine */
            smbtreeline = strip(smbtreeline,,'09'x)
            parse var smbtreeline '\\'machine '0909'x comment
            machine = left(strip(machine),13)
            comment = strip(comment)
            if target = "" then say '\\'machine' 'comment
        end
        otherwise do
            if pos("RECEIVING",translate(smbTreeline)) > 0 |,
               pos("TDB(",translate(smbTreeline)) > 0 then do
                Msg.Type = "W"
                Msg.Text = smbtreeline
            end
            else do
                if target = "" then do
                    say "Workgroup: "smbtreeline
                    say "Server          Comment"
                end
                else say "Share         Comment"
                say copies('-',79)
            end
        end
    end 
end
ok = stream(samba.!msg,'c','close')
ok = SysFileDelete(samba.!msg)
say 'Done.'
exit 0

_InitTempDir: /* make sure we have a valid temporary directory */
    /* Get temporary directory */
    HaveNoTMPDIR  = 0
    TempDir       = translate(value('TMPDIR',,'OS2ENVIRONMENT'),'\','/')
    if TempDir = '' then do
        HaveNoTMPDIR  = 1
        TempDir = translate(value('TEMP',,'OS2ENVIRONMENT'),'\','/')
    end
    if TempDir = '' then TempDir = translate(value('TMP',,'OS2ENVIRONMENT'),'\','/')
    if TempDir = '' then TempDir = directory()
    if HaveNoTMPDIR then do
        say 'WARNING! Setting missing TMPDIR   variable to "'TempDir'".'
        ok = value('TMPDIR',TempDir,'OS2ENVIRONMENT')
    end    
    TempDir       = strip(TempDir,'T','\')||'\' /* make sure there is trailing "\" */
return

_LoadOtherFuncs:
    '@echo off'

    call RxFuncAdd 'SysLoadFuncs', 'RexxUtil','SysLoadFuncs'
    call SysLoadFuncs
    
    call RxFuncAdd 'VRLoadFuncs',  'VROBJ',   'VRLoadFuncs'
    call VRLoadFuncs
return

_SambaInit: /* Detect Samba components, initialize samba. stem */
    ETC = value('ETC',,'OS2ENVIRONMENT')
    if ETC = "" then do
        say 'ETC environment variable is missing. Aborting...'
        exit -1
    end
    UnixRoot = value("UNIXROOT",,'OS2ENVIRONMENT')
    if UnixRoot = "" then do
        UnixRoot = left(ETC,length(ETC)-4)
        say 'WARNING! Setting missing UNIXROOT variable to "'UnixRoot'".'
        ok = value("UNIXROOT",UnixRoot,'OS2ENVIRONMENT')
    end
    UnixETC  = UnixRoot'\etc'

    /* Find out various directories */
    parse source . . smbcmd
    samba.=''
    samba.!tools = strip(filespec('D',smbcmd)||filespec('P',smbcmd),,'\')

    /* smbtree.exe */
    samba.!smbtreeexe = ""
    if samba.!smbtreeexe = "" then do /* in current directory? */
        ok = SysFileTree('.\smbtree.exe', exist.,'FO')
        if exist.0 = 1 then samba.!smbtreeexe = exist.1
    end
    if samba.!smbtreeexe = "" then do /* in parent directory? */
        ok = SysFileTree('..\smbtree.exe', exist.,'FO')
        if exist.0 = 1 then samba.!smbtreeexe = exist.1
    end
    if samba.!smbtreeexe = "" then do /* in ..\BIN directory? */
        ok = SysFileTree('..\bin\smbtree.exe', exist.,'FO')
        if exist.0 = 1 then samba.!smbtreeexe = exist.1
    end
    if samba.!smbtreeexe = "" then do /* in SMB_EXE ? */
        samba.!smbtreeexe = SysSearchPath("SMB_EXE","smbtree.exe")
    end
    if samba.!smbtreeexe = "" then do /* in PATH ? */
        samba.!smbtreeexe = SysSearchPath("PATH","smbtree.exe")
    end

    if samba.!smbtreeexe = "" then do
        say "Cannot find smbtree.exe - aborting..."
        exit -1
    end

    samba.!bin = VRParseFileName(samba.!smbtreeexe,'DP')

    if samba.!bin = "" then do
        say "Cannot find smbtree.exe - aborting..."
        exit -1
    end
    
    /* Check if smb.conf exists. */
    samba.!smbconf = stream(ETC'\samba\smb.conf','c','query exists')
    if samba.!smbconf = "" then do
        say ETC'\samba\smb.conf is missing. Aborting...'
        exit -2
    end
return
_SambaExtendSearchPath:
    /* Add binary and tools path to the PATH variable */
    old_path = value('PATH',, 'OS2ENVIRONMENT')
    if pos(translate(samba.!bin';'),translate(old_path)) = 0 then do
        if samba.!bin = samba.!tools then new_path = samba.!bin';'
        else new_path = samba.!bin';'samba.!tools';'
        ok = value('PATH', new_path || old_path, 'OS2ENVIRONMENT')
        drop new_path
    end
    drop old_path
    
    /* Add binary and tools path to the BEGINLIBPATH variable */
    old_beginlibpath = SysQueryEXtLibPath("B")
    if pos(translate(samba.!bin';'),translate(old_beginlibpath)) = 0 then do
        if samba.!bin = samba.!tools then new_beginlibpath = samba.!bin';'old_beginlibpath
        else new_beginlibpath = samba.!bin';'samba.!tools';'old_beginlibpath
        ok = SysSetExtLibPath( new_beginlibpath, "B")
        drop new_beginlibpath
    end
    drop old_beginlibpath
return
